/*
 * Copyright (c) 2007-2010 Sonatype, Inc. All rights reserved.
 *
 * "Sonatype" is a trademark of Sonatype, Inc.
 */

package org.sonatype.runtime.assembler;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import org.codehaus.plexus.util.FileUtils;
import org.codehaus.plexus.util.IOUtil;
import org.codehaus.plexus.util.SelectorUtils;
import org.codehaus.plexus.util.StringUtils;
import org.sonatype.plexus.template.Templater;
import org.sonatype.runtime.metadata.FileMode;
import org.sonatype.runtime.metadata.Metadata;
import org.sonatype.runtime.metadata.ProvidedProperty;
import org.sonatype.runtime.metadata.RequiredProperty;
import org.sonatype.runtime.metadata.io.xpp3.MetadataXpp3Reader;

import static java.lang.String.*;

/**
 * JAVADOC
 *
 * @author Alin Dreghiciu
 */
class Archive
{

    private final File archiveDir;
    private final Templater templater;

    private final Metadata metadata;
    private final List<AssemblyProperty> providedProperties;

    Archive( final File archiveDir,
             final Templater templater )
        throws Exception
    {
        this.archiveDir = archiveDir;
        this.templater = templater;
        this.providedProperties = new ArrayList<AssemblyProperty>();

        this.metadata = loadMetadata();
    }

    void expand( final File assemblyDir,
                 final AssemblyProperties assemblyProperties )
        throws Exception
    {
        final Map<String, Object> namespace = calculateNamespace( assemblyProperties );
        validateRequirements( namespace );
        expandDataDir( assemblyDir, new File( archiveDir, "data" ), assemblyDir, namespace );
        expandTemplatesDir( assemblyDir, new File( archiveDir, "templates" ), assemblyDir, namespace );
    }

    List<AssemblyProperty> getProvidedProperties()
    {
        return providedProperties;
    }

    void interpolate( final AssemblyProperties assemblyProperties )
        throws Exception
    {
        final AssemblyProperties localProperties = localProperties( assemblyProperties );

        // only replace properties provided by this archive
        for ( AssemblyProperty property : localProperties.get() )
        {
            final AssemblyProperty existing = assemblyProperties.get( property.getName() );
            // must be instance equality !
            if ( property == existing )
            {
                localProperties.interpolate( existing );
            }
        }

        localProperties.interpolate();
    }

    @Override
    public boolean equals( final Object o )
    {
        if ( this == o )
        {
            return true;
        }
        if ( !( o instanceof Archive ) )
        {
            return false;
        }

        final Archive archive = (Archive) o;

        if ( !archiveDir.equals( archive.archiveDir ) )
        {
            return false;
        }

        return true;
    }

    @Override
    public int hashCode()
    {
        return archiveDir.hashCode();
    }

    private Metadata loadMetadata()
        throws Exception
    {
        final File metadataXml = new File( archiveDir, "metadata/metadata.xml" );
        if ( !metadataXml.exists() )
        {
            throw new Exception( format( "Could not find metadata/metadata.xml in %s", archiveDir.getPath() ) );
        }

        final FileReader reader = new FileReader( metadataXml );
        final Metadata metadata = new MetadataXpp3Reader().read( reader );
        IOUtil.close( reader );

        if ( metadata.getProvides() != null )
        {
            for ( ProvidedProperty providedProperty : metadata.getProvides().getProperty() )
            {
                AssemblyProperty property;
                if ( providedProperty.getValue() != null )
                {
                    property = new AssemblyProperty(
                        providedProperty.getName(),
                        providedProperty.getValue(),
                        MergePolicy.valueOf( providedProperty.getMergePolicy().toUpperCase() )
                    );
                }
                else
                {
                    if ( providedProperty.getValues().getListDirectory() != null )
                    {
                        // TODO handle case when path does ends with /
                        List<String> files = listDirectory(
                            providedProperty.getValues().getListDirectory() + "/",
                            new File( archiveDir, "data/" + providedProperty.getValues().getListDirectory() )
                        );
                        property = new AssemblyProperty(
                            providedProperty.getName(),
                            files.toArray( new String[files.size()] ),
                            MergePolicy.valueOf( providedProperty.getMergePolicy().toUpperCase() )
                        );
                    }
                    else
                    {
                        final List<String> values = providedProperty.getValues().getValue();
                        property = new AssemblyProperty(
                            providedProperty.getName(),
                            values.toArray( new String[values.size()] ),
                            MergePolicy.valueOf( providedProperty.getMergePolicy().toUpperCase() )
                        );
                    }
                }
                providedProperties.add( property );
            }
        }

        return metadata;
    }

    private Map<String, Object> calculateNamespace( final AssemblyProperties assemblyProperties )
        throws Exception
    {
        final AssemblyProperties localProperties = localProperties( assemblyProperties );

        localProperties.interpolate();

        return localProperties.asMap();
    }

    private AssemblyProperties localProperties( final AssemblyProperties assemblyProperties )
        throws Exception
    {
        final AssemblyProperties localProperties = assemblyProperties.clone();
        if ( metadata.getRequires() != null
             && metadata.getRequires().getProperty() != null )
        {
            for ( RequiredProperty property : metadata.getRequires().getProperty() )
            {
                if ( property.isOptional()
                     && ( property.getDefaultValue() != null )
                     && !assemblyProperties.contains( property.getName() ) )
                {
                    localProperties.add(
                        new AssemblyProperty(
                            property.getName(),
                            property.getDefaultValue(),
                            MergePolicy.LAST
                        )
                    );
                }
            }
        }
        return localProperties;
    }

    private void validateRequirements( final Map<String, Object> namespace )
        throws Exception
    {
        if ( metadata.getRequires() == null
             || metadata.getRequires().getProperty() == null )
        {
            return;
        }

        for ( RequiredProperty required : metadata.getRequires().getProperty() )
        {
            if ( !namespace.containsKey( required.getName() )
                 && !required.isOptional() )
            {
                throw new Exception(
                    format(
                        "Required property [%s] must be defined. "
                        + "Property is required by [%s]",
                        required.getName(),
                        metadata.getDescription()
                    )
                );
            }
        }

    }

    private void expandDataDir( final File assemblyDir,
                                final File sourceDir,
                                final File targetDir,
                                final Map<String, Object> namespace )
        throws Exception
    {
        // TODO shall we use Plexus utils to walk the directory
        if ( sourceDir.exists() && sourceDir.canRead() )
        {
            final File[] files = sourceDir.listFiles();
            for ( File file : files )
            {
                final String targetName = StringUtils.interpolate( file.getName(), namespace );
                final File targetFile = new File( targetDir, targetName );

                if ( file.isDirectory() )
                {
                    expandDataDir( assemblyDir, file, targetFile, namespace );
                }
                else
                {
                    FileUtils.copyFile( file, targetFile );
                    changeFileMode( assemblyDir, targetFile, namespace );
                }
            }
        }
    }

    private void expandTemplatesDir( final File assemblyDir,
                                     final File sourceDir,
                                     final File targetDir,
                                     final Map<String, Object> namespace )
        throws Exception
    {
        // TODO shall we use Plexus utils to walk the directory
        if ( sourceDir.exists() )
        {
            final File[] files = sourceDir.listFiles();
            for ( File file : files )
            {
                final String targetName = StringUtils.interpolate( file.getName(), namespace );
                final File targetFile = new File( targetDir, targetName );

                if ( file.isDirectory() )
                {
                    expandTemplatesDir( assemblyDir, file, targetFile, namespace );
                }
                else
                {
                    renderTemplate( file, targetFile, namespace );
                    changeFileMode( assemblyDir, targetFile, namespace );
                }
            }
        }
    }

    private void renderTemplate( final File source,
                                 final File target,
                                 final Map<String, Object> namespace )
        throws Exception
    {
        if ( !target.getParentFile().exists() )
        {
            target.getParentFile().mkdirs();
        }
        Writer writer = new FileWriter( target );
        templater.renderTemplate( source.getPath(), namespace, writer );
        IOUtil.close( writer );
    }

    private void changeFileMode( final File targetDir,
                                 final File targetFile,
                                 final Map<String, Object> namespace )
        throws Exception
    {
        if ( metadata.getFileModes() == null || metadata.getFileModes().isEmpty() )
        {
            return;
        }

        for ( FileMode fileMode : metadata.getFileModes() )
        {
            for ( String path : fileMode.getPaths() )
            {
                final String realPath = targetDir.getPath() + "/" + StringUtils.interpolate( path, namespace );
                if ( SelectorUtils.matchPath( realPath, targetFile.getPath() ) )
                {
                    FileModeUtils.chmod( targetFile, FileModeUtils.modeToInt( fileMode.getMode() ) );
                }
            }
        }
    }

    private static List<String> listDirectory( final String path,
                                               final File directory )
    {
        if ( directory != null && directory.exists() )
        {
            List<String> files = new ArrayList<String>();
            final File[] content = directory.listFiles();
            for ( File file : content )
            {
                if ( file.isDirectory() )
                {
                    // TODO handle case when path does not end with /
                    files.addAll( listDirectory( path + file.getName() + "/", file ) );
                }
                else
                {
                    files.add( path + file.getName() );
                }
            }
            return files;
        }
        return Collections.emptyList();
    }

}
