/*
 * Copyright (c) 2007-2010 Sonatype, Inc. All rights reserved.
 *
 * "Sonatype" is a trademark of Sonatype, Inc.
 */

package org.sonatype.runtime.assembler;

import java.io.File;
import java.io.FileReader;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import org.sonatype.plexus.template.Templater;
import org.sonatype.runtime.descriptor.AssembliesDescriptor;
import org.sonatype.runtime.descriptor.AssemblyDescriptor;
import org.sonatype.runtime.descriptor.AssemblyDescriptorArtifact;
import org.sonatype.runtime.descriptor.AssemblyDescriptorItarArtifact;
import org.sonatype.runtime.descriptor.InheritedAssembliesDescriptor;
import org.sonatype.runtime.descriptor.io.xpp3.AssembliesDescriptorXpp3Reader;

import static java.lang.String.*;

/**
 * JAVADOC
 *
 * @author Alin Dreghiciu
 */
public class Assembly
{

    private final ArtifactResolver resolver;
    private final Templater templater;

    private final Set<AssemblyProperty> descriptorProperties;
    private final Set<Archive> archives;

    private final AssembliesDescriptorXpp3Reader descriptorReader;

    Assembly( final AssemblyDescriptor descriptor,
              final ArtifactResolver resolver,
              final Templater templater )
        throws Exception
    {
        this.resolver = resolver;
        this.templater = templater;

        this.descriptorProperties = new HashSet<AssemblyProperty>();
        this.archives = new HashSet<Archive>();

        this.descriptorReader = new AssembliesDescriptorXpp3Reader();

        loadDescriptor( descriptor );
    }

    public void build( final Map<String, String> envProperties,
                       final File targetDir )
        throws Exception
    {
        final AssemblyProperties assemblyProperties = composeProperties( envProperties );

        for ( Archive archive : archives )
        {
            archive.expand( targetDir, assemblyProperties );
        }
    }

    private AssemblyProperties composeProperties( final Map<String, String> envProperties )
        throws Exception
    {
        final AssemblyProperties assemblyProperties = new AssemblyProperties();

        if ( envProperties != null )
        {
            for ( Map.Entry<String, String> entry : envProperties.entrySet() )
            {
                assemblyProperties.add( new AssemblyProperty( entry.getKey(), entry.getValue(), MergePolicy.LAST ) );
            }
        }

        for ( AssemblyProperty property : descriptorProperties )
        {
            assemblyProperties.add( property );
        }

        for ( Archive archive : archives )
        {
            assemblyProperties.add( archive.getProvidedProperties() );
        }

        assemblyProperties.interpolate();

        for ( Archive archive : archives )
        {
            archive.interpolate( assemblyProperties );
        }

        return assemblyProperties;
    }

    private void loadDescriptor( final AssemblyDescriptor descriptor )
        throws Exception
    {
        if ( descriptor.getProperties() != null )
        {
            for ( Map.Entry<String, String> entry : ( (Map<String, String>) descriptor.getProperties() ).entrySet() )
            {
                descriptorProperties.add( new AssemblyProperty( entry.getKey(), entry.getValue(), MergePolicy.LAST ) );
            }
        }

        if ( descriptor.getBar() != null )
        {
            archives.add( new Archive( resolver.resolve( descriptor.getBar(), true ), templater ) );
        }
        if ( descriptor.getCars() != null )
        {
            for ( AssemblyDescriptorArtifact car : descriptor.getCars() )
            {
                archives.add( new Archive( resolver.resolve( car, true ), templater ) );
            }
        }
        if ( descriptor.getItars() != null )
        {
            for ( AssemblyDescriptorItarArtifact itar : descriptor.getItars() )
            {
                if ( itar.isVirtualItar() )
                {
                    archives.add(
                        new VirtualArchive(
                            resolver.resolve( itar, itar.isUnpack() ),
                            templater,
                            itar.getOutputDirectory()
                        )
                    );
                }
                else
                {
                    archives.add( new Archive( resolver.resolve( itar, true ), templater ) );
                }
            }
        }

        loadMixins( descriptor );
    }

    private void loadMixins( final AssemblyDescriptor descriptor )
        throws Exception
    {
        if ( descriptor.getExtended() == null || descriptor.getExtended().isEmpty() )
        {
            return;
        }
        for ( InheritedAssembliesDescriptor mixin : descriptor.getExtended() )
        {
            final File mixinXML = resolver.resolve( mixin, false );
            final AssembliesDescriptor mixinDescriptor = descriptorReader.read( new FileReader( mixinXML ) );
            if ( mixinDescriptor.getAssemblies() == null )
            {
                throw new Exception(
                    format( "Inherited descriptor %s does not contain any assembly", toString( mixin ) )
                );
            }
            if ( mixin.getAssemblyId() == null )
            {
                if ( mixinDescriptor.getAssemblies().size() > 1 )
                {
                    throw new Exception(
                        format(
                            "Assembly id has not been specified for %s, which contains more then one assembly ",
                            toString( mixin )
                        )
                    );
                }
                loadDescriptor( mixinDescriptor.getAssemblies().get( 0 ) );
            }
            else
            {
                for ( AssemblyDescriptor inherited : mixinDescriptor.getAssemblies() )
                {
                    if ( mixin.getAssemblyId().equals( inherited.getId() ) )
                    {
                        loadDescriptor( inherited );
                        break;
                    }
                }
            }
        }
    }

    private static String toString( final AssemblyDescriptorArtifact artifact )
    {
        return new StringBuilder()
            .append( artifact.getGroupId() )
            .append( ":" )
            .append( artifact.getArtifactId() )
            .append( ":" )
            .append( artifact.getVersion() )
            .append( ":" )
            .append( artifact.getClassifier() == null ? "" : artifact.getClassifier() )
            .append( ":" )
            .append( artifact.getType() == null ? "" : artifact.getType() )
            .toString();
    }


}