/*
 * Copyright (c) 2007-2010 Sonatype, Inc. All rights reserved.
 *
 * "Sonatype" is a trademark of Sonatype, Inc.
 */

package org.sonatype.runtime.assembler;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import org.codehaus.plexus.util.StringUtils;

/**
 * JAVADOC
 *
 * @author Alin Dreghiciu
 */
public class AssemblyProperties
    implements Cloneable
{

    private Map<String, AssemblyProperty> properties;

    AssemblyProperties()
    {
        properties = new HashMap<String, AssemblyProperty>();
    }

    AssemblyProperties add( final Collection<AssemblyProperty> properties )
        throws Exception
    {
        for ( AssemblyProperty property : properties )
        {
            add( property );
        }

        return this;
    }

    AssemblyProperties add( final AssemblyProperty property )
        throws Exception
    {
        final AssemblyProperty existing = get( property.getName() );
        if ( existing == null )
        {
            properties.put( property.getName(), property );
        }
        else
        {
            switch ( property.getMergePolicy() )
            {
                case ERROR:
                    throw new Exception(
                        String.format( "Property %s already present with value %s",
                                       existing.getName(),
                                       Arrays.deepToString( existing.getValues() )
                        )
                    );
                case FIRST:
                    // Keep teh existing value
                    break;
                case LAST:
                    properties.put( property.getName(), property );
                case APPEND:
                    String[] values = new String[existing.getValues().length + property.getValues().length];
                    System.arraycopy(
                        existing.getValues(), 0, values, 0, existing.getValues().length
                    );
                    System.arraycopy(
                        property.getValues(), 0, values, existing.getValues().length, property.getValues().length
                    );
                    properties.put(
                        property.getName(),
                        new AssemblyProperty( property.getName(), values, MergePolicy.APPEND )
                    );
            }
        }

        return this;
    }

    boolean contains( final String name )
    {
        return properties.containsKey( name );
    }

    AssemblyProperty get( final String name )
    {
        return properties.get( name );
    }

    Collection<AssemblyProperty> get()
    {
        return properties.values();
    }

    Map<String, Object> asMap()
    {
        final Map<String, Object> map = new HashMap<String, Object>();
        for ( Map.Entry<String, AssemblyProperty> entry : properties.entrySet() )
        {
            if ( entry.getValue().isCollection() )
            {
                map.put( entry.getKey(), entry.getValue().getValues() );
            }
            else
            {
                // TODO avoid NPE on values array
                map.put( entry.getKey(), entry.getValue().getValues()[ 0 ] );
            }
        }

        return map;
    }

    AssemblyProperties interpolate()
    {
        for ( AssemblyProperty property : properties.values() )
        {
            for ( AssemblyProperty target : properties.values() )
            {
                Map<String, Object> namespace = new HashMap<String, Object>();
                if ( property.isCollection() )
                {
                    namespace.put( property.getName(), property.getValues() );
                }
                else
                {
                    // TODO avoid NPE on values array
                    namespace.put( property.getName(), property.getValues()[ 0 ] );
                }

                final String[] values = target.getValues();
                for ( int i = 0; i < values.length; i++ )
                {
                    values[ i ] = StringUtils.interpolate( values[ i ], namespace );
                }
            }
        }

        return this;
    }

    AssemblyProperties interpolate( final AssemblyProperty target )
    {
        for ( AssemblyProperty property : properties.values() )
        {
            Map<String, Object> namespace = new HashMap<String, Object>();
            if ( property.isCollection() )
            {
                namespace.put( property.getName(), property.getValues() );
            }
            else
            {
                // TODO avoid NPE on values array
                namespace.put( property.getName(), property.getValues()[ 0 ] );
            }

            final String[] values = target.getValues();
            for ( int i = 0; i < values.length; i++ )
            {
                values[ i ] = StringUtils.interpolate( values[ i ], namespace );
            }
        }

        return this;
    }

    void list()
    {
        System.out.println( "--------------------------" );
        for ( AssemblyProperty property : properties.values() )
        {
            System.out.println( property.getName() + " -> " + Arrays.deepToString( property.getValues() ) );
        }
        System.out.println( "--------------------------" );
    }

    @Override
    protected AssemblyProperties clone()
        throws CloneNotSupportedException
    {
        super.clone();

        final AssemblyProperties cloned = new AssemblyProperties();

        cloned.properties.putAll( properties );

        return cloned;
    }

}
