/*
 * Copyright (c) 2007-2010 Sonatype, Inc. All rights reserved.
 *
 * "Sonatype" is a trademark of Sonatype, Inc.
 */

package org.sonatype.runtime.assembler;

import java.io.File;
import org.codehaus.plexus.util.Os;
import org.codehaus.plexus.util.cli.CommandLineException;
import org.codehaus.plexus.util.cli.CommandLineUtils;
import org.codehaus.plexus.util.cli.Commandline;

/**
 * JAVADOC
 *
 * @author Alin Dreghiciu
 */
final class FileModeUtils
{

    private static final int U_R = 256;
    private static final int U_W = 128;
    private static final int U_X = 64;

    private static final int G_R = 32;
    private static final int G_W = 16;
    private static final int G_X = 8;

    private static final int W_R = 4;
    private static final int W_W = 2;
    private static final int W_X = 1;

    private FileModeUtils()
    {
    }

    static void chmod( File file, int mode )
        throws Exception
    {
        if ( !Os.isFamily( Os.FAMILY_UNIX ) )
        {
            return;
        }
        String m = Integer.toOctalString( mode & 0xfff );
        try
        {
            Commandline commandline = new Commandline();
            commandline.setWorkingDirectory( file.getParentFile().getAbsolutePath() );
            commandline.setExecutable( "chmod" );
            commandline.createArg().setValue( m );
            String path = file.getAbsolutePath();
            commandline.createArg().setValue( path );
            CommandLineUtils.StringStreamConsumer stderr = new CommandLineUtils.StringStreamConsumer();
            CommandLineUtils.StringStreamConsumer stdout = new CommandLineUtils.StringStreamConsumer();
            int exitCode = CommandLineUtils.executeCommandLine( commandline, stderr, stdout );
            if ( exitCode != 0 )
            {
                // TODO USe some logger?
                System.err.println( "-------------------------------" );
                System.err.println( "Standard error:" );
                System.err.println( "-------------------------------" );
                System.err.println( stderr.getOutput() );
                System.err.println( "-------------------------------" );
                System.err.println( "Standard output:" );
                System.err.println( "-------------------------------" );
                System.err.println( stdout.getOutput() );
                System.err.println( "-------------------------------" );
                throw new Exception( "chmod exit code was: " + exitCode );
            }
        }
        catch ( CommandLineException e )
        {
            throw new Exception( "Error while executing chmod.", e );
        }
    }

    static int modeToInt( String mode )
        throws Exception
    {
        if ( mode == null || mode.trim().length() < 1 )
        {
            return -1;
        }

        try
        {
            int value = Integer.parseInt( mode, 8 );

            // discard sanity assessment here; we're pushing ahead.
            verifyModeSanity( value );

            return value;
        }
        catch ( NumberFormatException e )
        {
            throw new Exception( "Failed to parse mode as an octal number: \'" + mode + "\'.", e );
        }
    }

    // the boolean return type is for people who want to make a decision based on the sanity
    // assessment.
    public static boolean verifyModeSanity( int mode )
    {
        StringBuffer messages = new StringBuffer();

        messages.append( "The mode: " + Integer.toString( mode, 8 ) + " contains nonsensical permissions:" );

        boolean warn = false;

        // read-access checks.
        if ( ( ( mode & U_R ) == 0 ) && ( ( mode & G_R ) == G_R ) )
        {
            messages.append( "\n- Group has read access, but user does not." );
            warn = true;
        }

        if ( ( ( mode & U_R ) == 0 ) && ( ( mode & W_R ) == W_R ) )
        {
            messages.append( "\n- World has read access, but user does not." );
            warn = true;
        }

        if ( ( ( mode & G_R ) == 0 ) && ( ( mode & W_R ) == W_R ) )
        {
            messages.append( "\n- World has read access, but group does not." );
            warn = true;
        }
        // end read-access checks.

        // write-access checks.
        if ( ( ( mode & U_W ) == 0 ) && ( ( mode & G_W ) == G_W ) )
        {
            messages.append( "\n- Group has write access, but user does not." );
            warn = true;
        }

        if ( ( ( mode & U_W ) == 0 ) && ( ( mode & W_W ) == W_W ) )
        {
            messages.append( "\n- World has write access, but user does not." );
            warn = true;
        }

        if ( ( ( mode & G_W ) == 0 ) && ( ( mode & W_W ) == W_W ) )
        {
            messages.append( "\n- World has write access, but group does not." );
            warn = true;
        }
        // end write-access checks.

        // execute-/list-access checks.
        if ( ( ( mode & U_X ) == 0 ) && ( ( mode & G_X ) == G_X ) )
        {
            messages.append( "\n- Group has execute/list access, but user does not." );
            warn = true;
        }

        if ( ( ( mode & U_X ) == 0 ) && ( ( mode & W_X ) == W_X ) )
        {
            messages.append( "\n- World has execute/list access, but user does not." );
            warn = true;
        }

        if ( ( ( mode & G_X ) == 0 ) && ( ( mode & W_X ) == W_X ) )
        {
            messages.append( "\n- World has execute/list access, but group does not." );
            warn = true;
        }
        // end execute-/list-access checks.

        if ( warn )
        {
            // TODO use some kind of logger?
            System.err.println( messages.toString() );
        }

        return !warn;
    }

}
